# Generated by the gRPC Python protocol compiler plugin. DO NOT EDIT!
"""Client and server classes corresponding to protobuf-defined services."""
import grpc

from common.api.services.common import common_pb2 as services_dot_common_dot_common__pb2
from common.api.services.product import product_pb2 as services_dot_product_dot_product__pb2


class ProductServiceStub(object):
    """Missing associated documentation comment in .proto file."""

    def __init__(self, channel):
        """Constructor.

        Args:
            channel: A grpc.Channel.
        """
        self.Ping = channel.unary_unary(
                '/social.product.ProductService/Ping',
                request_serializer=services_dot_common_dot_common__pb2.Empty.SerializeToString,
                response_deserializer=services_dot_common_dot_common__pb2.Pong.FromString,
                )
        self.Publish = channel.unary_unary(
                '/social.product.ProductService/Publish',
                request_serializer=services_dot_product_dot_product__pb2.Product.SerializeToString,
                response_deserializer=services_dot_product_dot_product__pb2.Product.FromString,
                )
        self.Update = channel.unary_unary(
                '/social.product.ProductService/Update',
                request_serializer=services_dot_product_dot_product__pb2.Product.SerializeToString,
                response_deserializer=services_dot_product_dot_product__pb2.Product.FromString,
                )
        self.Delete = channel.unary_unary(
                '/social.product.ProductService/Delete',
                request_serializer=services_dot_product_dot_product__pb2.Product.SerializeToString,
                response_deserializer=services_dot_common_dot_common__pb2.Empty.FromString,
                )
        self.Get = channel.unary_unary(
                '/social.product.ProductService/Get',
                request_serializer=services_dot_product_dot_product__pb2.GetProduct.SerializeToString,
                response_deserializer=services_dot_product_dot_product__pb2.Product.FromString,
                )
        self.Find = channel.unary_unary(
                '/social.product.ProductService/Find',
                request_serializer=services_dot_product_dot_product__pb2.FindProductRequest.SerializeToString,
                response_deserializer=services_dot_product_dot_product__pb2.ProductList.FromString,
                )
        self.GetAll = channel.unary_unary(
                '/social.product.ProductService/GetAll',
                request_serializer=services_dot_product_dot_product__pb2.GetAllRequest.SerializeToString,
                response_deserializer=services_dot_product_dot_product__pb2.ProductList.FromString,
                )
        self.GetMyProducts = channel.unary_unary(
                '/social.product.ProductService/GetMyProducts',
                request_serializer=services_dot_common_dot_common__pb2.Pagination.SerializeToString,
                response_deserializer=services_dot_product_dot_product__pb2.ProductList.FromString,
                )
        self.GetAllCategories = channel.unary_unary(
                '/social.product.ProductService/GetAllCategories',
                request_serializer=services_dot_product_dot_product__pb2.GetAllCategoriesRequest.SerializeToString,
                response_deserializer=services_dot_product_dot_product__pb2.ProductCategoryList.FromString,
                )
        self.GetCategory = channel.unary_unary(
                '/social.product.ProductService/GetCategory',
                request_serializer=services_dot_product_dot_product__pb2.GetCategoryRequest.SerializeToString,
                response_deserializer=services_dot_product_dot_product__pb2.ProductCategory.FromString,
                )
        self.GetAllCategoriesWithSub = channel.unary_unary(
                '/social.product.ProductService/GetAllCategoriesWithSub',
                request_serializer=services_dot_product_dot_product__pb2.GetAllCategoriesWithSubRequest.SerializeToString,
                response_deserializer=services_dot_product_dot_product__pb2.ProductCategoryList.FromString,
                )
        self.FindCategory = channel.unary_unary(
                '/social.product.ProductService/FindCategory',
                request_serializer=services_dot_product_dot_product__pb2.FindCategoryRequest.SerializeToString,
                response_deserializer=services_dot_product_dot_product__pb2.ProductCategoryList.FromString,
                )
        self.UploadMedia = channel.unary_unary(
                '/social.product.ProductService/UploadMedia',
                request_serializer=services_dot_common_dot_common__pb2.UploadMediaRequest.SerializeToString,
                response_deserializer=services_dot_common_dot_common__pb2.UploadMediaResponse.FromString,
                )
        self.UploadMediaCategory = channel.unary_unary(
                '/social.product.ProductService/UploadMediaCategory',
                request_serializer=services_dot_common_dot_common__pb2.UploadMediaRequest.SerializeToString,
                response_deserializer=services_dot_common_dot_common__pb2.UploadMediaResponse.FromString,
                )
        self.UploadStyleThumbnail = channel.unary_unary(
                '/social.product.ProductService/UploadStyleThumbnail',
                request_serializer=services_dot_common_dot_common__pb2.UploadMediaRequest.SerializeToString,
                response_deserializer=services_dot_common_dot_common__pb2.UploadMediaResponse.FromString,
                )
        self.GetRootCategory = channel.unary_unary(
                '/social.product.ProductService/GetRootCategory',
                request_serializer=services_dot_common_dot_common__pb2.Empty.SerializeToString,
                response_deserializer=services_dot_product_dot_product__pb2.RootCategory.FromString,
                )
        self.GetSubCategoryContent = channel.unary_unary(
                '/social.product.ProductService/GetSubCategoryContent',
                request_serializer=services_dot_product_dot_product__pb2.ProductCategory.SerializeToString,
                response_deserializer=services_dot_product_dot_product__pb2.SubCategoryContent.FromString,
                )
        self.GetCategoryProperty = channel.unary_unary(
                '/social.product.ProductService/GetCategoryProperty',
                request_serializer=services_dot_product_dot_product__pb2.ProductCategory.SerializeToString,
                response_deserializer=services_dot_product_dot_product__pb2.PropertyList.FromString,
                )
        self.CreateStyle = channel.unary_unary(
                '/social.product.ProductService/CreateStyle',
                request_serializer=services_dot_product_dot_product__pb2.StyleMaker.SerializeToString,
                response_deserializer=services_dot_product_dot_product__pb2.StyleMaker.FromString,
                )
        self.UpdateStyle = channel.unary_unary(
                '/social.product.ProductService/UpdateStyle',
                request_serializer=services_dot_product_dot_product__pb2.StyleMaker.SerializeToString,
                response_deserializer=services_dot_product_dot_product__pb2.StyleMaker.FromString,
                )
        self.DeleteStyle = channel.unary_unary(
                '/social.product.ProductService/DeleteStyle',
                request_serializer=services_dot_product_dot_product__pb2.StyleMaker.SerializeToString,
                response_deserializer=services_dot_common_dot_common__pb2.Empty.FromString,
                )
        self.ArchiveStyle = channel.unary_unary(
                '/social.product.ProductService/ArchiveStyle',
                request_serializer=services_dot_product_dot_product__pb2.StyleMaker.SerializeToString,
                response_deserializer=services_dot_common_dot_common__pb2.Empty.FromString,
                )
        self.PublicStyle = channel.unary_unary(
                '/social.product.ProductService/PublicStyle',
                request_serializer=services_dot_product_dot_product__pb2.StyleMaker.SerializeToString,
                response_deserializer=services_dot_common_dot_common__pb2.Empty.FromString,
                )
        self.GetStyle = channel.unary_unary(
                '/social.product.ProductService/GetStyle',
                request_serializer=services_dot_product_dot_product__pb2.StyleMaker.SerializeToString,
                response_deserializer=services_dot_product_dot_product__pb2.StyleMaker.FromString,
                )
        self.GetAllStyles = channel.unary_unary(
                '/social.product.ProductService/GetAllStyles',
                request_serializer=services_dot_product_dot_product__pb2.GetAllStylesRequest.SerializeToString,
                response_deserializer=services_dot_product_dot_product__pb2.StyleMakerList.FromString,
                )
        self.GetFavorite = channel.unary_unary(
                '/social.product.ProductService/GetFavorite',
                request_serializer=services_dot_common_dot_common__pb2.Pagination.SerializeToString,
                response_deserializer=services_dot_product_dot_product__pb2.ProductList.FromString,
                )
        self.AddToFavorite = channel.unary_unary(
                '/social.product.ProductService/AddToFavorite',
                request_serializer=services_dot_product_dot_product__pb2.Product.SerializeToString,
                response_deserializer=services_dot_common_dot_common__pb2.Empty.FromString,
                )
        self.RemoveFromFavorite = channel.unary_unary(
                '/social.product.ProductService/RemoveFromFavorite',
                request_serializer=services_dot_product_dot_product__pb2.Product.SerializeToString,
                response_deserializer=services_dot_common_dot_common__pb2.Empty.FromString,
                )
        self.PublishReview = channel.unary_unary(
                '/social.product.ProductService/PublishReview',
                request_serializer=services_dot_product_dot_product__pb2.Review.SerializeToString,
                response_deserializer=services_dot_product_dot_product__pb2.Review.FromString,
                )
        self.UpdateReview = channel.unary_unary(
                '/social.product.ProductService/UpdateReview',
                request_serializer=services_dot_product_dot_product__pb2.Review.SerializeToString,
                response_deserializer=services_dot_product_dot_product__pb2.Review.FromString,
                )
        self.DeleteReview = channel.unary_unary(
                '/social.product.ProductService/DeleteReview',
                request_serializer=services_dot_product_dot_product__pb2.Review.SerializeToString,
                response_deserializer=services_dot_common_dot_common__pb2.Empty.FromString,
                )
        self.GetAllReviews = channel.unary_unary(
                '/social.product.ProductService/GetAllReviews',
                request_serializer=services_dot_product_dot_product__pb2.GetAllReviewsRequest.SerializeToString,
                response_deserializer=services_dot_product_dot_product__pb2.ReviewList.FromString,
                )
        self.GetUnApprovedReviews = channel.unary_unary(
                '/social.product.ProductService/GetUnApprovedReviews',
                request_serializer=services_dot_product_dot_product__pb2.GetAllReviewsRequest.SerializeToString,
                response_deserializer=services_dot_product_dot_product__pb2.ReviewList.FromString,
                )
        self.GetReview = channel.unary_unary(
                '/social.product.ProductService/GetReview',
                request_serializer=services_dot_product_dot_product__pb2.Review.SerializeToString,
                response_deserializer=services_dot_product_dot_product__pb2.Review.FromString,
                )
        self.AdminGetAll = channel.unary_unary(
                '/social.product.ProductService/AdminGetAll',
                request_serializer=services_dot_common_dot_common__pb2.Pagination.SerializeToString,
                response_deserializer=services_dot_product_dot_product__pb2.ProductList.FromString,
                )
        self.AdminApproveProduct = channel.unary_unary(
                '/social.product.ProductService/AdminApproveProduct',
                request_serializer=services_dot_product_dot_product__pb2.ProductPublishStatusRequest.SerializeToString,
                response_deserializer=services_dot_common_dot_common__pb2.Empty.FromString,
                )
        self.AdminRejectProduct = channel.unary_unary(
                '/social.product.ProductService/AdminRejectProduct',
                request_serializer=services_dot_product_dot_product__pb2.ProductPublishStatusRequest.SerializeToString,
                response_deserializer=services_dot_common_dot_common__pb2.Empty.FromString,
                )
        self.AdminGetUnapprovedProduct = channel.unary_unary(
                '/social.product.ProductService/AdminGetUnapprovedProduct',
                request_serializer=services_dot_common_dot_common__pb2.Pagination.SerializeToString,
                response_deserializer=services_dot_product_dot_product__pb2.ProductList.FromString,
                )
        self.AdminApproveReview = channel.unary_unary(
                '/social.product.ProductService/AdminApproveReview',
                request_serializer=services_dot_product_dot_product__pb2.ReviewPublishStatusRequest.SerializeToString,
                response_deserializer=services_dot_common_dot_common__pb2.Empty.FromString,
                )
        self.AdminRejectReview = channel.unary_unary(
                '/social.product.ProductService/AdminRejectReview',
                request_serializer=services_dot_product_dot_product__pb2.ReviewPublishStatusRequest.SerializeToString,
                response_deserializer=services_dot_common_dot_common__pb2.Empty.FromString,
                )
        self.AdminGetUnapprovedReview = channel.unary_unary(
                '/social.product.ProductService/AdminGetUnapprovedReview',
                request_serializer=services_dot_common_dot_common__pb2.Pagination.SerializeToString,
                response_deserializer=services_dot_product_dot_product__pb2.ReviewList.FromString,
                )
        self.AdminGetReviewByStatus = channel.unary_unary(
                '/social.product.ProductService/AdminGetReviewByStatus',
                request_serializer=services_dot_product_dot_product__pb2.GetReviewsRequest.SerializeToString,
                response_deserializer=services_dot_product_dot_product__pb2.ReviewList.FromString,
                )
        self.AdminCreateProperty = channel.unary_unary(
                '/social.product.ProductService/AdminCreateProperty',
                request_serializer=services_dot_product_dot_product__pb2.Property.SerializeToString,
                response_deserializer=services_dot_product_dot_product__pb2.Property.FromString,
                )
        self.AdminUpdateProperty = channel.unary_unary(
                '/social.product.ProductService/AdminUpdateProperty',
                request_serializer=services_dot_product_dot_product__pb2.Property.SerializeToString,
                response_deserializer=services_dot_product_dot_product__pb2.Property.FromString,
                )
        self.AdminDeleteProperty = channel.unary_unary(
                '/social.product.ProductService/AdminDeleteProperty',
                request_serializer=services_dot_product_dot_product__pb2.Property.SerializeToString,
                response_deserializer=services_dot_common_dot_common__pb2.Empty.FromString,
                )
        self.AdminGetAllProperties = channel.unary_unary(
                '/social.product.ProductService/AdminGetAllProperties',
                request_serializer=services_dot_common_dot_common__pb2.Pagination.SerializeToString,
                response_deserializer=services_dot_product_dot_product__pb2.PropertyList.FromString,
                )
        self.AdminFindProperty = channel.unary_unary(
                '/social.product.ProductService/AdminFindProperty',
                request_serializer=services_dot_common_dot_common__pb2.FindRequest.SerializeToString,
                response_deserializer=services_dot_product_dot_product__pb2.PropertyList.FromString,
                )
        self.AdminCreateCategory = channel.unary_unary(
                '/social.product.ProductService/AdminCreateCategory',
                request_serializer=services_dot_product_dot_product__pb2.ProductCategory.SerializeToString,
                response_deserializer=services_dot_product_dot_product__pb2.ProductCategory.FromString,
                )
        self.AdminUpdateCategory = channel.unary_unary(
                '/social.product.ProductService/AdminUpdateCategory',
                request_serializer=services_dot_product_dot_product__pb2.ProductCategory.SerializeToString,
                response_deserializer=services_dot_product_dot_product__pb2.ProductCategory.FromString,
                )
        self.AdminDeleteCategory = channel.unary_unary(
                '/social.product.ProductService/AdminDeleteCategory',
                request_serializer=services_dot_product_dot_product__pb2.ProductCategory.SerializeToString,
                response_deserializer=services_dot_common_dot_common__pb2.Empty.FromString,
                )
        self.AdminGetCategory = channel.unary_unary(
                '/social.product.ProductService/AdminGetCategory',
                request_serializer=services_dot_product_dot_product__pb2.GetCategoryRequest.SerializeToString,
                response_deserializer=services_dot_product_dot_product__pb2.ProductCategory.FromString,
                )


class ProductServiceServicer(object):
    """Missing associated documentation comment in .proto file."""

    def Ping(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def Publish(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def Update(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def Delete(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def Get(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def Find(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetAll(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetMyProducts(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetAllCategories(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetCategory(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetAllCategoriesWithSub(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def FindCategory(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def UploadMedia(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def UploadMediaCategory(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def UploadStyleThumbnail(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetRootCategory(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetSubCategoryContent(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetCategoryProperty(self, request, context):
        """Property
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def CreateStyle(self, request, context):
        """StyleMaker
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def UpdateStyle(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def DeleteStyle(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def ArchiveStyle(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def PublicStyle(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetStyle(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetAllStyles(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetFavorite(self, request, context):
        """Closet
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def AddToFavorite(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def RemoveFromFavorite(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def PublishReview(self, request, context):
        """Review
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def UpdateReview(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def DeleteReview(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetAllReviews(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetUnApprovedReviews(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetReview(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def AdminGetAll(self, request, context):
        """Admin

        Product
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def AdminApproveProduct(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def AdminRejectProduct(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def AdminGetUnapprovedProduct(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def AdminApproveReview(self, request, context):
        """Review
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def AdminRejectReview(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def AdminGetUnapprovedReview(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def AdminGetReviewByStatus(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def AdminCreateProperty(self, request, context):
        """Property
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def AdminUpdateProperty(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def AdminDeleteProperty(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def AdminGetAllProperties(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def AdminFindProperty(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def AdminCreateCategory(self, request, context):
        """Category
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def AdminUpdateCategory(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def AdminDeleteCategory(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def AdminGetCategory(self, request, context):
        """Get category by id
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')


def add_ProductServiceServicer_to_server(servicer, server):
    rpc_method_handlers = {
            'Ping': grpc.unary_unary_rpc_method_handler(
                    servicer.Ping,
                    request_deserializer=services_dot_common_dot_common__pb2.Empty.FromString,
                    response_serializer=services_dot_common_dot_common__pb2.Pong.SerializeToString,
            ),
            'Publish': grpc.unary_unary_rpc_method_handler(
                    servicer.Publish,
                    request_deserializer=services_dot_product_dot_product__pb2.Product.FromString,
                    response_serializer=services_dot_product_dot_product__pb2.Product.SerializeToString,
            ),
            'Update': grpc.unary_unary_rpc_method_handler(
                    servicer.Update,
                    request_deserializer=services_dot_product_dot_product__pb2.Product.FromString,
                    response_serializer=services_dot_product_dot_product__pb2.Product.SerializeToString,
            ),
            'Delete': grpc.unary_unary_rpc_method_handler(
                    servicer.Delete,
                    request_deserializer=services_dot_product_dot_product__pb2.Product.FromString,
                    response_serializer=services_dot_common_dot_common__pb2.Empty.SerializeToString,
            ),
            'Get': grpc.unary_unary_rpc_method_handler(
                    servicer.Get,
                    request_deserializer=services_dot_product_dot_product__pb2.GetProduct.FromString,
                    response_serializer=services_dot_product_dot_product__pb2.Product.SerializeToString,
            ),
            'Find': grpc.unary_unary_rpc_method_handler(
                    servicer.Find,
                    request_deserializer=services_dot_product_dot_product__pb2.FindProductRequest.FromString,
                    response_serializer=services_dot_product_dot_product__pb2.ProductList.SerializeToString,
            ),
            'GetAll': grpc.unary_unary_rpc_method_handler(
                    servicer.GetAll,
                    request_deserializer=services_dot_product_dot_product__pb2.GetAllRequest.FromString,
                    response_serializer=services_dot_product_dot_product__pb2.ProductList.SerializeToString,
            ),
            'GetMyProducts': grpc.unary_unary_rpc_method_handler(
                    servicer.GetMyProducts,
                    request_deserializer=services_dot_common_dot_common__pb2.Pagination.FromString,
                    response_serializer=services_dot_product_dot_product__pb2.ProductList.SerializeToString,
            ),
            'GetAllCategories': grpc.unary_unary_rpc_method_handler(
                    servicer.GetAllCategories,
                    request_deserializer=services_dot_product_dot_product__pb2.GetAllCategoriesRequest.FromString,
                    response_serializer=services_dot_product_dot_product__pb2.ProductCategoryList.SerializeToString,
            ),
            'GetCategory': grpc.unary_unary_rpc_method_handler(
                    servicer.GetCategory,
                    request_deserializer=services_dot_product_dot_product__pb2.GetCategoryRequest.FromString,
                    response_serializer=services_dot_product_dot_product__pb2.ProductCategory.SerializeToString,
            ),
            'GetAllCategoriesWithSub': grpc.unary_unary_rpc_method_handler(
                    servicer.GetAllCategoriesWithSub,
                    request_deserializer=services_dot_product_dot_product__pb2.GetAllCategoriesWithSubRequest.FromString,
                    response_serializer=services_dot_product_dot_product__pb2.ProductCategoryList.SerializeToString,
            ),
            'FindCategory': grpc.unary_unary_rpc_method_handler(
                    servicer.FindCategory,
                    request_deserializer=services_dot_product_dot_product__pb2.FindCategoryRequest.FromString,
                    response_serializer=services_dot_product_dot_product__pb2.ProductCategoryList.SerializeToString,
            ),
            'UploadMedia': grpc.unary_unary_rpc_method_handler(
                    servicer.UploadMedia,
                    request_deserializer=services_dot_common_dot_common__pb2.UploadMediaRequest.FromString,
                    response_serializer=services_dot_common_dot_common__pb2.UploadMediaResponse.SerializeToString,
            ),
            'UploadMediaCategory': grpc.unary_unary_rpc_method_handler(
                    servicer.UploadMediaCategory,
                    request_deserializer=services_dot_common_dot_common__pb2.UploadMediaRequest.FromString,
                    response_serializer=services_dot_common_dot_common__pb2.UploadMediaResponse.SerializeToString,
            ),
            'UploadStyleThumbnail': grpc.unary_unary_rpc_method_handler(
                    servicer.UploadStyleThumbnail,
                    request_deserializer=services_dot_common_dot_common__pb2.UploadMediaRequest.FromString,
                    response_serializer=services_dot_common_dot_common__pb2.UploadMediaResponse.SerializeToString,
            ),
            'GetRootCategory': grpc.unary_unary_rpc_method_handler(
                    servicer.GetRootCategory,
                    request_deserializer=services_dot_common_dot_common__pb2.Empty.FromString,
                    response_serializer=services_dot_product_dot_product__pb2.RootCategory.SerializeToString,
            ),
            'GetSubCategoryContent': grpc.unary_unary_rpc_method_handler(
                    servicer.GetSubCategoryContent,
                    request_deserializer=services_dot_product_dot_product__pb2.ProductCategory.FromString,
                    response_serializer=services_dot_product_dot_product__pb2.SubCategoryContent.SerializeToString,
            ),
            'GetCategoryProperty': grpc.unary_unary_rpc_method_handler(
                    servicer.GetCategoryProperty,
                    request_deserializer=services_dot_product_dot_product__pb2.ProductCategory.FromString,
                    response_serializer=services_dot_product_dot_product__pb2.PropertyList.SerializeToString,
            ),
            'CreateStyle': grpc.unary_unary_rpc_method_handler(
                    servicer.CreateStyle,
                    request_deserializer=services_dot_product_dot_product__pb2.StyleMaker.FromString,
                    response_serializer=services_dot_product_dot_product__pb2.StyleMaker.SerializeToString,
            ),
            'UpdateStyle': grpc.unary_unary_rpc_method_handler(
                    servicer.UpdateStyle,
                    request_deserializer=services_dot_product_dot_product__pb2.StyleMaker.FromString,
                    response_serializer=services_dot_product_dot_product__pb2.StyleMaker.SerializeToString,
            ),
            'DeleteStyle': grpc.unary_unary_rpc_method_handler(
                    servicer.DeleteStyle,
                    request_deserializer=services_dot_product_dot_product__pb2.StyleMaker.FromString,
                    response_serializer=services_dot_common_dot_common__pb2.Empty.SerializeToString,
            ),
            'ArchiveStyle': grpc.unary_unary_rpc_method_handler(
                    servicer.ArchiveStyle,
                    request_deserializer=services_dot_product_dot_product__pb2.StyleMaker.FromString,
                    response_serializer=services_dot_common_dot_common__pb2.Empty.SerializeToString,
            ),
            'PublicStyle': grpc.unary_unary_rpc_method_handler(
                    servicer.PublicStyle,
                    request_deserializer=services_dot_product_dot_product__pb2.StyleMaker.FromString,
                    response_serializer=services_dot_common_dot_common__pb2.Empty.SerializeToString,
            ),
            'GetStyle': grpc.unary_unary_rpc_method_handler(
                    servicer.GetStyle,
                    request_deserializer=services_dot_product_dot_product__pb2.StyleMaker.FromString,
                    response_serializer=services_dot_product_dot_product__pb2.StyleMaker.SerializeToString,
            ),
            'GetAllStyles': grpc.unary_unary_rpc_method_handler(
                    servicer.GetAllStyles,
                    request_deserializer=services_dot_product_dot_product__pb2.GetAllStylesRequest.FromString,
                    response_serializer=services_dot_product_dot_product__pb2.StyleMakerList.SerializeToString,
            ),
            'GetFavorite': grpc.unary_unary_rpc_method_handler(
                    servicer.GetFavorite,
                    request_deserializer=services_dot_common_dot_common__pb2.Pagination.FromString,
                    response_serializer=services_dot_product_dot_product__pb2.ProductList.SerializeToString,
            ),
            'AddToFavorite': grpc.unary_unary_rpc_method_handler(
                    servicer.AddToFavorite,
                    request_deserializer=services_dot_product_dot_product__pb2.Product.FromString,
                    response_serializer=services_dot_common_dot_common__pb2.Empty.SerializeToString,
            ),
            'RemoveFromFavorite': grpc.unary_unary_rpc_method_handler(
                    servicer.RemoveFromFavorite,
                    request_deserializer=services_dot_product_dot_product__pb2.Product.FromString,
                    response_serializer=services_dot_common_dot_common__pb2.Empty.SerializeToString,
            ),
            'PublishReview': grpc.unary_unary_rpc_method_handler(
                    servicer.PublishReview,
                    request_deserializer=services_dot_product_dot_product__pb2.Review.FromString,
                    response_serializer=services_dot_product_dot_product__pb2.Review.SerializeToString,
            ),
            'UpdateReview': grpc.unary_unary_rpc_method_handler(
                    servicer.UpdateReview,
                    request_deserializer=services_dot_product_dot_product__pb2.Review.FromString,
                    response_serializer=services_dot_product_dot_product__pb2.Review.SerializeToString,
            ),
            'DeleteReview': grpc.unary_unary_rpc_method_handler(
                    servicer.DeleteReview,
                    request_deserializer=services_dot_product_dot_product__pb2.Review.FromString,
                    response_serializer=services_dot_common_dot_common__pb2.Empty.SerializeToString,
            ),
            'GetAllReviews': grpc.unary_unary_rpc_method_handler(
                    servicer.GetAllReviews,
                    request_deserializer=services_dot_product_dot_product__pb2.GetAllReviewsRequest.FromString,
                    response_serializer=services_dot_product_dot_product__pb2.ReviewList.SerializeToString,
            ),
            'GetUnApprovedReviews': grpc.unary_unary_rpc_method_handler(
                    servicer.GetUnApprovedReviews,
                    request_deserializer=services_dot_product_dot_product__pb2.GetAllReviewsRequest.FromString,
                    response_serializer=services_dot_product_dot_product__pb2.ReviewList.SerializeToString,
            ),
            'GetReview': grpc.unary_unary_rpc_method_handler(
                    servicer.GetReview,
                    request_deserializer=services_dot_product_dot_product__pb2.Review.FromString,
                    response_serializer=services_dot_product_dot_product__pb2.Review.SerializeToString,
            ),
            'AdminGetAll': grpc.unary_unary_rpc_method_handler(
                    servicer.AdminGetAll,
                    request_deserializer=services_dot_common_dot_common__pb2.Pagination.FromString,
                    response_serializer=services_dot_product_dot_product__pb2.ProductList.SerializeToString,
            ),
            'AdminApproveProduct': grpc.unary_unary_rpc_method_handler(
                    servicer.AdminApproveProduct,
                    request_deserializer=services_dot_product_dot_product__pb2.ProductPublishStatusRequest.FromString,
                    response_serializer=services_dot_common_dot_common__pb2.Empty.SerializeToString,
            ),
            'AdminRejectProduct': grpc.unary_unary_rpc_method_handler(
                    servicer.AdminRejectProduct,
                    request_deserializer=services_dot_product_dot_product__pb2.ProductPublishStatusRequest.FromString,
                    response_serializer=services_dot_common_dot_common__pb2.Empty.SerializeToString,
            ),
            'AdminGetUnapprovedProduct': grpc.unary_unary_rpc_method_handler(
                    servicer.AdminGetUnapprovedProduct,
                    request_deserializer=services_dot_common_dot_common__pb2.Pagination.FromString,
                    response_serializer=services_dot_product_dot_product__pb2.ProductList.SerializeToString,
            ),
            'AdminApproveReview': grpc.unary_unary_rpc_method_handler(
                    servicer.AdminApproveReview,
                    request_deserializer=services_dot_product_dot_product__pb2.ReviewPublishStatusRequest.FromString,
                    response_serializer=services_dot_common_dot_common__pb2.Empty.SerializeToString,
            ),
            'AdminRejectReview': grpc.unary_unary_rpc_method_handler(
                    servicer.AdminRejectReview,
                    request_deserializer=services_dot_product_dot_product__pb2.ReviewPublishStatusRequest.FromString,
                    response_serializer=services_dot_common_dot_common__pb2.Empty.SerializeToString,
            ),
            'AdminGetUnapprovedReview': grpc.unary_unary_rpc_method_handler(
                    servicer.AdminGetUnapprovedReview,
                    request_deserializer=services_dot_common_dot_common__pb2.Pagination.FromString,
                    response_serializer=services_dot_product_dot_product__pb2.ReviewList.SerializeToString,
            ),
            'AdminGetReviewByStatus': grpc.unary_unary_rpc_method_handler(
                    servicer.AdminGetReviewByStatus,
                    request_deserializer=services_dot_product_dot_product__pb2.GetReviewsRequest.FromString,
                    response_serializer=services_dot_product_dot_product__pb2.ReviewList.SerializeToString,
            ),
            'AdminCreateProperty': grpc.unary_unary_rpc_method_handler(
                    servicer.AdminCreateProperty,
                    request_deserializer=services_dot_product_dot_product__pb2.Property.FromString,
                    response_serializer=services_dot_product_dot_product__pb2.Property.SerializeToString,
            ),
            'AdminUpdateProperty': grpc.unary_unary_rpc_method_handler(
                    servicer.AdminUpdateProperty,
                    request_deserializer=services_dot_product_dot_product__pb2.Property.FromString,
                    response_serializer=services_dot_product_dot_product__pb2.Property.SerializeToString,
            ),
            'AdminDeleteProperty': grpc.unary_unary_rpc_method_handler(
                    servicer.AdminDeleteProperty,
                    request_deserializer=services_dot_product_dot_product__pb2.Property.FromString,
                    response_serializer=services_dot_common_dot_common__pb2.Empty.SerializeToString,
            ),
            'AdminGetAllProperties': grpc.unary_unary_rpc_method_handler(
                    servicer.AdminGetAllProperties,
                    request_deserializer=services_dot_common_dot_common__pb2.Pagination.FromString,
                    response_serializer=services_dot_product_dot_product__pb2.PropertyList.SerializeToString,
            ),
            'AdminFindProperty': grpc.unary_unary_rpc_method_handler(
                    servicer.AdminFindProperty,
                    request_deserializer=services_dot_common_dot_common__pb2.FindRequest.FromString,
                    response_serializer=services_dot_product_dot_product__pb2.PropertyList.SerializeToString,
            ),
            'AdminCreateCategory': grpc.unary_unary_rpc_method_handler(
                    servicer.AdminCreateCategory,
                    request_deserializer=services_dot_product_dot_product__pb2.ProductCategory.FromString,
                    response_serializer=services_dot_product_dot_product__pb2.ProductCategory.SerializeToString,
            ),
            'AdminUpdateCategory': grpc.unary_unary_rpc_method_handler(
                    servicer.AdminUpdateCategory,
                    request_deserializer=services_dot_product_dot_product__pb2.ProductCategory.FromString,
                    response_serializer=services_dot_product_dot_product__pb2.ProductCategory.SerializeToString,
            ),
            'AdminDeleteCategory': grpc.unary_unary_rpc_method_handler(
                    servicer.AdminDeleteCategory,
                    request_deserializer=services_dot_product_dot_product__pb2.ProductCategory.FromString,
                    response_serializer=services_dot_common_dot_common__pb2.Empty.SerializeToString,
            ),
            'AdminGetCategory': grpc.unary_unary_rpc_method_handler(
                    servicer.AdminGetCategory,
                    request_deserializer=services_dot_product_dot_product__pb2.GetCategoryRequest.FromString,
                    response_serializer=services_dot_product_dot_product__pb2.ProductCategory.SerializeToString,
            ),
    }
    generic_handler = grpc.method_handlers_generic_handler(
            'social.product.ProductService', rpc_method_handlers)
    server.add_generic_rpc_handlers((generic_handler,))


 # This class is part of an EXPERIMENTAL API.
class ProductService(object):
    """Missing associated documentation comment in .proto file."""

    @staticmethod
    def Ping(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/social.product.ProductService/Ping',
            services_dot_common_dot_common__pb2.Empty.SerializeToString,
            services_dot_common_dot_common__pb2.Pong.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def Publish(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/social.product.ProductService/Publish',
            services_dot_product_dot_product__pb2.Product.SerializeToString,
            services_dot_product_dot_product__pb2.Product.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def Update(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/social.product.ProductService/Update',
            services_dot_product_dot_product__pb2.Product.SerializeToString,
            services_dot_product_dot_product__pb2.Product.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def Delete(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/social.product.ProductService/Delete',
            services_dot_product_dot_product__pb2.Product.SerializeToString,
            services_dot_common_dot_common__pb2.Empty.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def Get(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/social.product.ProductService/Get',
            services_dot_product_dot_product__pb2.GetProduct.SerializeToString,
            services_dot_product_dot_product__pb2.Product.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def Find(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/social.product.ProductService/Find',
            services_dot_product_dot_product__pb2.FindProductRequest.SerializeToString,
            services_dot_product_dot_product__pb2.ProductList.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def GetAll(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/social.product.ProductService/GetAll',
            services_dot_product_dot_product__pb2.GetAllRequest.SerializeToString,
            services_dot_product_dot_product__pb2.ProductList.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def GetMyProducts(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/social.product.ProductService/GetMyProducts',
            services_dot_common_dot_common__pb2.Pagination.SerializeToString,
            services_dot_product_dot_product__pb2.ProductList.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def GetAllCategories(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/social.product.ProductService/GetAllCategories',
            services_dot_product_dot_product__pb2.GetAllCategoriesRequest.SerializeToString,
            services_dot_product_dot_product__pb2.ProductCategoryList.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def GetCategory(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/social.product.ProductService/GetCategory',
            services_dot_product_dot_product__pb2.GetCategoryRequest.SerializeToString,
            services_dot_product_dot_product__pb2.ProductCategory.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def GetAllCategoriesWithSub(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/social.product.ProductService/GetAllCategoriesWithSub',
            services_dot_product_dot_product__pb2.GetAllCategoriesWithSubRequest.SerializeToString,
            services_dot_product_dot_product__pb2.ProductCategoryList.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def FindCategory(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/social.product.ProductService/FindCategory',
            services_dot_product_dot_product__pb2.FindCategoryRequest.SerializeToString,
            services_dot_product_dot_product__pb2.ProductCategoryList.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def UploadMedia(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/social.product.ProductService/UploadMedia',
            services_dot_common_dot_common__pb2.UploadMediaRequest.SerializeToString,
            services_dot_common_dot_common__pb2.UploadMediaResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def UploadMediaCategory(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/social.product.ProductService/UploadMediaCategory',
            services_dot_common_dot_common__pb2.UploadMediaRequest.SerializeToString,
            services_dot_common_dot_common__pb2.UploadMediaResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def UploadStyleThumbnail(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/social.product.ProductService/UploadStyleThumbnail',
            services_dot_common_dot_common__pb2.UploadMediaRequest.SerializeToString,
            services_dot_common_dot_common__pb2.UploadMediaResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def GetRootCategory(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/social.product.ProductService/GetRootCategory',
            services_dot_common_dot_common__pb2.Empty.SerializeToString,
            services_dot_product_dot_product__pb2.RootCategory.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def GetSubCategoryContent(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/social.product.ProductService/GetSubCategoryContent',
            services_dot_product_dot_product__pb2.ProductCategory.SerializeToString,
            services_dot_product_dot_product__pb2.SubCategoryContent.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def GetCategoryProperty(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/social.product.ProductService/GetCategoryProperty',
            services_dot_product_dot_product__pb2.ProductCategory.SerializeToString,
            services_dot_product_dot_product__pb2.PropertyList.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def CreateStyle(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/social.product.ProductService/CreateStyle',
            services_dot_product_dot_product__pb2.StyleMaker.SerializeToString,
            services_dot_product_dot_product__pb2.StyleMaker.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def UpdateStyle(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/social.product.ProductService/UpdateStyle',
            services_dot_product_dot_product__pb2.StyleMaker.SerializeToString,
            services_dot_product_dot_product__pb2.StyleMaker.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def DeleteStyle(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/social.product.ProductService/DeleteStyle',
            services_dot_product_dot_product__pb2.StyleMaker.SerializeToString,
            services_dot_common_dot_common__pb2.Empty.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def ArchiveStyle(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/social.product.ProductService/ArchiveStyle',
            services_dot_product_dot_product__pb2.StyleMaker.SerializeToString,
            services_dot_common_dot_common__pb2.Empty.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def PublicStyle(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/social.product.ProductService/PublicStyle',
            services_dot_product_dot_product__pb2.StyleMaker.SerializeToString,
            services_dot_common_dot_common__pb2.Empty.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def GetStyle(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/social.product.ProductService/GetStyle',
            services_dot_product_dot_product__pb2.StyleMaker.SerializeToString,
            services_dot_product_dot_product__pb2.StyleMaker.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def GetAllStyles(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/social.product.ProductService/GetAllStyles',
            services_dot_product_dot_product__pb2.GetAllStylesRequest.SerializeToString,
            services_dot_product_dot_product__pb2.StyleMakerList.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def GetFavorite(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/social.product.ProductService/GetFavorite',
            services_dot_common_dot_common__pb2.Pagination.SerializeToString,
            services_dot_product_dot_product__pb2.ProductList.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def AddToFavorite(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/social.product.ProductService/AddToFavorite',
            services_dot_product_dot_product__pb2.Product.SerializeToString,
            services_dot_common_dot_common__pb2.Empty.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def RemoveFromFavorite(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/social.product.ProductService/RemoveFromFavorite',
            services_dot_product_dot_product__pb2.Product.SerializeToString,
            services_dot_common_dot_common__pb2.Empty.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def PublishReview(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/social.product.ProductService/PublishReview',
            services_dot_product_dot_product__pb2.Review.SerializeToString,
            services_dot_product_dot_product__pb2.Review.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def UpdateReview(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/social.product.ProductService/UpdateReview',
            services_dot_product_dot_product__pb2.Review.SerializeToString,
            services_dot_product_dot_product__pb2.Review.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def DeleteReview(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/social.product.ProductService/DeleteReview',
            services_dot_product_dot_product__pb2.Review.SerializeToString,
            services_dot_common_dot_common__pb2.Empty.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def GetAllReviews(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/social.product.ProductService/GetAllReviews',
            services_dot_product_dot_product__pb2.GetAllReviewsRequest.SerializeToString,
            services_dot_product_dot_product__pb2.ReviewList.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def GetUnApprovedReviews(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/social.product.ProductService/GetUnApprovedReviews',
            services_dot_product_dot_product__pb2.GetAllReviewsRequest.SerializeToString,
            services_dot_product_dot_product__pb2.ReviewList.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def GetReview(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/social.product.ProductService/GetReview',
            services_dot_product_dot_product__pb2.Review.SerializeToString,
            services_dot_product_dot_product__pb2.Review.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def AdminGetAll(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/social.product.ProductService/AdminGetAll',
            services_dot_common_dot_common__pb2.Pagination.SerializeToString,
            services_dot_product_dot_product__pb2.ProductList.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def AdminApproveProduct(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/social.product.ProductService/AdminApproveProduct',
            services_dot_product_dot_product__pb2.ProductPublishStatusRequest.SerializeToString,
            services_dot_common_dot_common__pb2.Empty.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def AdminRejectProduct(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/social.product.ProductService/AdminRejectProduct',
            services_dot_product_dot_product__pb2.ProductPublishStatusRequest.SerializeToString,
            services_dot_common_dot_common__pb2.Empty.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def AdminGetUnapprovedProduct(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/social.product.ProductService/AdminGetUnapprovedProduct',
            services_dot_common_dot_common__pb2.Pagination.SerializeToString,
            services_dot_product_dot_product__pb2.ProductList.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def AdminApproveReview(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/social.product.ProductService/AdminApproveReview',
            services_dot_product_dot_product__pb2.ReviewPublishStatusRequest.SerializeToString,
            services_dot_common_dot_common__pb2.Empty.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def AdminRejectReview(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/social.product.ProductService/AdminRejectReview',
            services_dot_product_dot_product__pb2.ReviewPublishStatusRequest.SerializeToString,
            services_dot_common_dot_common__pb2.Empty.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def AdminGetUnapprovedReview(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/social.product.ProductService/AdminGetUnapprovedReview',
            services_dot_common_dot_common__pb2.Pagination.SerializeToString,
            services_dot_product_dot_product__pb2.ReviewList.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def AdminGetReviewByStatus(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/social.product.ProductService/AdminGetReviewByStatus',
            services_dot_product_dot_product__pb2.GetReviewsRequest.SerializeToString,
            services_dot_product_dot_product__pb2.ReviewList.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def AdminCreateProperty(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/social.product.ProductService/AdminCreateProperty',
            services_dot_product_dot_product__pb2.Property.SerializeToString,
            services_dot_product_dot_product__pb2.Property.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def AdminUpdateProperty(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/social.product.ProductService/AdminUpdateProperty',
            services_dot_product_dot_product__pb2.Property.SerializeToString,
            services_dot_product_dot_product__pb2.Property.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def AdminDeleteProperty(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/social.product.ProductService/AdminDeleteProperty',
            services_dot_product_dot_product__pb2.Property.SerializeToString,
            services_dot_common_dot_common__pb2.Empty.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def AdminGetAllProperties(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/social.product.ProductService/AdminGetAllProperties',
            services_dot_common_dot_common__pb2.Pagination.SerializeToString,
            services_dot_product_dot_product__pb2.PropertyList.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def AdminFindProperty(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/social.product.ProductService/AdminFindProperty',
            services_dot_common_dot_common__pb2.FindRequest.SerializeToString,
            services_dot_product_dot_product__pb2.PropertyList.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def AdminCreateCategory(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/social.product.ProductService/AdminCreateCategory',
            services_dot_product_dot_product__pb2.ProductCategory.SerializeToString,
            services_dot_product_dot_product__pb2.ProductCategory.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def AdminUpdateCategory(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/social.product.ProductService/AdminUpdateCategory',
            services_dot_product_dot_product__pb2.ProductCategory.SerializeToString,
            services_dot_product_dot_product__pb2.ProductCategory.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def AdminDeleteCategory(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/social.product.ProductService/AdminDeleteCategory',
            services_dot_product_dot_product__pb2.ProductCategory.SerializeToString,
            services_dot_common_dot_common__pb2.Empty.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def AdminGetCategory(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/social.product.ProductService/AdminGetCategory',
            services_dot_product_dot_product__pb2.GetCategoryRequest.SerializeToString,
            services_dot_product_dot_product__pb2.ProductCategory.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)



# Generated by the gRPC Python protocol compiler plugin. DO NOT EDIT!
"""Client and server classes corresponding to protobuf-defined services."""
import grpc

from common.api.services.account import account_pb2 as services_dot_account_dot_account__pb2
from common.api.services.common import common_pb2 as services_dot_common_dot_common__pb2
from common.api.services.hashtag import hashtag_pb2 as services_dot_hashtag_dot_hashtag__pb2


class HashtagServiceStub(object):
    """Missing associated documentation comment in .proto file."""

    def __init__(self, channel):
        """Constructor.

        Args:
            channel: A grpc.Channel.
        """
        self.Ping = channel.unary_unary(
                '/social.hashtag.HashtagService/Ping',
                request_serializer=services_dot_common_dot_common__pb2.Empty.SerializeToString,
                response_deserializer=services_dot_common_dot_common__pb2.Pong.FromString,
                )
        self.Find = channel.unary_unary(
                '/social.hashtag.HashtagService/Find',
                request_serializer=services_dot_common_dot_common__pb2.FindRequest.SerializeToString,
                response_deserializer=services_dot_hashtag_dot_hashtag__pb2.FindResponse.FromString,
                )
        self.Follow = channel.unary_unary(
                '/social.hashtag.HashtagService/Follow',
                request_serializer=services_dot_hashtag_dot_hashtag__pb2.Hashtag.SerializeToString,
                response_deserializer=services_dot_common_dot_common__pb2.Empty.FromString,
                )
        self.Unfollow = channel.unary_unary(
                '/social.hashtag.HashtagService/Unfollow',
                request_serializer=services_dot_hashtag_dot_hashtag__pb2.Hashtag.SerializeToString,
                response_deserializer=services_dot_common_dot_common__pb2.Empty.FromString,
                )
        self.GetFeed = channel.unary_unary(
                '/social.hashtag.HashtagService/GetFeed',
                request_serializer=services_dot_hashtag_dot_hashtag__pb2.GetFeedRequest.SerializeToString,
                response_deserializer=services_dot_hashtag_dot_hashtag__pb2.GetFeedResponse.FromString,
                )
        self.GetHashtagFollowing = channel.unary_unary(
                '/social.hashtag.HashtagService/GetHashtagFollowing',
                request_serializer=services_dot_account_dot_account__pb2.AccountListRequest.SerializeToString,
                response_deserializer=services_dot_hashtag_dot_hashtag__pb2.GetHashtagFollowingResponse.FromString,
                )
        self.AdminHashtagDelete = channel.unary_unary(
                '/social.hashtag.HashtagService/AdminHashtagDelete',
                request_serializer=services_dot_hashtag_dot_hashtag__pb2.ToggleHashtag.SerializeToString,
                response_deserializer=services_dot_common_dot_common__pb2.Empty.FromString,
                )
        self.AdminHashtagBan = channel.unary_unary(
                '/social.hashtag.HashtagService/AdminHashtagBan',
                request_serializer=services_dot_hashtag_dot_hashtag__pb2.ToggleHashtag.SerializeToString,
                response_deserializer=services_dot_common_dot_common__pb2.Empty.FromString,
                )


class HashtagServiceServicer(object):
    """Missing associated documentation comment in .proto file."""

    def Ping(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def Find(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def Follow(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def Unfollow(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetFeed(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetHashtagFollowing(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def AdminHashtagDelete(self, request, context):
        """Admin Section. All admin services starts with `AdminSERVICE_NAME`
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def AdminHashtagBan(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')


def add_HashtagServiceServicer_to_server(servicer, server):
    rpc_method_handlers = {
            'Ping': grpc.unary_unary_rpc_method_handler(
                    servicer.Ping,
                    request_deserializer=services_dot_common_dot_common__pb2.Empty.FromString,
                    response_serializer=services_dot_common_dot_common__pb2.Pong.SerializeToString,
            ),
            'Find': grpc.unary_unary_rpc_method_handler(
                    servicer.Find,
                    request_deserializer=services_dot_common_dot_common__pb2.FindRequest.FromString,
                    response_serializer=services_dot_hashtag_dot_hashtag__pb2.FindResponse.SerializeToString,
            ),
            'Follow': grpc.unary_unary_rpc_method_handler(
                    servicer.Follow,
                    request_deserializer=services_dot_hashtag_dot_hashtag__pb2.Hashtag.FromString,
                    response_serializer=services_dot_common_dot_common__pb2.Empty.SerializeToString,
            ),
            'Unfollow': grpc.unary_unary_rpc_method_handler(
                    servicer.Unfollow,
                    request_deserializer=services_dot_hashtag_dot_hashtag__pb2.Hashtag.FromString,
                    response_serializer=services_dot_common_dot_common__pb2.Empty.SerializeToString,
            ),
            'GetFeed': grpc.unary_unary_rpc_method_handler(
                    servicer.GetFeed,
                    request_deserializer=services_dot_hashtag_dot_hashtag__pb2.GetFeedRequest.FromString,
                    response_serializer=services_dot_hashtag_dot_hashtag__pb2.GetFeedResponse.SerializeToString,
            ),
            'GetHashtagFollowing': grpc.unary_unary_rpc_method_handler(
                    servicer.GetHashtagFollowing,
                    request_deserializer=services_dot_account_dot_account__pb2.AccountListRequest.FromString,
                    response_serializer=services_dot_hashtag_dot_hashtag__pb2.GetHashtagFollowingResponse.SerializeToString,
            ),
            'AdminHashtagDelete': grpc.unary_unary_rpc_method_handler(
                    servicer.AdminHashtagDelete,
                    request_deserializer=services_dot_hashtag_dot_hashtag__pb2.ToggleHashtag.FromString,
                    response_serializer=services_dot_common_dot_common__pb2.Empty.SerializeToString,
            ),
            'AdminHashtagBan': grpc.unary_unary_rpc_method_handler(
                    servicer.AdminHashtagBan,
                    request_deserializer=services_dot_hashtag_dot_hashtag__pb2.ToggleHashtag.FromString,
                    response_serializer=services_dot_common_dot_common__pb2.Empty.SerializeToString,
            ),
    }
    generic_handler = grpc.method_handlers_generic_handler(
            'social.hashtag.HashtagService', rpc_method_handlers)
    server.add_generic_rpc_handlers((generic_handler,))


 # This class is part of an EXPERIMENTAL API.
class HashtagService(object):
    """Missing associated documentation comment in .proto file."""

    @staticmethod
    def Ping(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/social.hashtag.HashtagService/Ping',
            services_dot_common_dot_common__pb2.Empty.SerializeToString,
            services_dot_common_dot_common__pb2.Pong.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def Find(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/social.hashtag.HashtagService/Find',
            services_dot_common_dot_common__pb2.FindRequest.SerializeToString,
            services_dot_hashtag_dot_hashtag__pb2.FindResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def Follow(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/social.hashtag.HashtagService/Follow',
            services_dot_hashtag_dot_hashtag__pb2.Hashtag.SerializeToString,
            services_dot_common_dot_common__pb2.Empty.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def Unfollow(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/social.hashtag.HashtagService/Unfollow',
            services_dot_hashtag_dot_hashtag__pb2.Hashtag.SerializeToString,
            services_dot_common_dot_common__pb2.Empty.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def GetFeed(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/social.hashtag.HashtagService/GetFeed',
            services_dot_hashtag_dot_hashtag__pb2.GetFeedRequest.SerializeToString,
            services_dot_hashtag_dot_hashtag__pb2.GetFeedResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def GetHashtagFollowing(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/social.hashtag.HashtagService/GetHashtagFollowing',
            services_dot_account_dot_account__pb2.AccountListRequest.SerializeToString,
            services_dot_hashtag_dot_hashtag__pb2.GetHashtagFollowingResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def AdminHashtagDelete(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/social.hashtag.HashtagService/AdminHashtagDelete',
            services_dot_hashtag_dot_hashtag__pb2.ToggleHashtag.SerializeToString,
            services_dot_common_dot_common__pb2.Empty.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def AdminHashtagBan(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/social.hashtag.HashtagService/AdminHashtagBan',
            services_dot_hashtag_dot_hashtag__pb2.ToggleHashtag.SerializeToString,
            services_dot_common_dot_common__pb2.Empty.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)



# Generated by the gRPC Python protocol compiler plugin. DO NOT EDIT!
"""Client and server classes corresponding to protobuf-defined services."""
import grpc

from common.api.services.common import common_pb2 as services_dot_common_dot_common__pb2
from common.api.services.explore import explore_pb2 as services_dot_explore_dot_explore__pb2
from common.api.services.post import post_pb2 as services_dot_post_dot_post__pb2


class ExploreServiceStub(object):
    """Missing associated documentation comment in .proto file."""

    def __init__(self, channel):
        """Constructor.

        Args:
            channel: A grpc.Channel.
        """
        self.Ping = channel.unary_unary(
                '/social.explore.ExploreService/Ping',
                request_serializer=services_dot_common_dot_common__pb2.Empty.SerializeToString,
                response_deserializer=services_dot_common_dot_common__pb2.Pong.FromString,
                )
        self.GetFeed = channel.unary_unary(
                '/social.explore.ExploreService/GetFeed',
                request_serializer=services_dot_common_dot_common__pb2.Pagination.SerializeToString,
                response_deserializer=services_dot_post_dot_post__pb2.PostListResponse.FromString,
                )
        self.GeoSearch = channel.unary_unary(
                '/social.explore.ExploreService/GeoSearch',
                request_serializer=services_dot_explore_dot_explore__pb2.GeoSearchRequest.SerializeToString,
                response_deserializer=services_dot_explore_dot_explore__pb2.GeoSearchResponse.FromString,
                )


class ExploreServiceServicer(object):
    """Missing associated documentation comment in .proto file."""

    def Ping(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetFeed(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GeoSearch(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')


def add_ExploreServiceServicer_to_server(servicer, server):
    rpc_method_handlers = {
            'Ping': grpc.unary_unary_rpc_method_handler(
                    servicer.Ping,
                    request_deserializer=services_dot_common_dot_common__pb2.Empty.FromString,
                    response_serializer=services_dot_common_dot_common__pb2.Pong.SerializeToString,
            ),
            'GetFeed': grpc.unary_unary_rpc_method_handler(
                    servicer.GetFeed,
                    request_deserializer=services_dot_common_dot_common__pb2.Pagination.FromString,
                    response_serializer=services_dot_post_dot_post__pb2.PostListResponse.SerializeToString,
            ),
            'GeoSearch': grpc.unary_unary_rpc_method_handler(
                    servicer.GeoSearch,
                    request_deserializer=services_dot_explore_dot_explore__pb2.GeoSearchRequest.FromString,
                    response_serializer=services_dot_explore_dot_explore__pb2.GeoSearchResponse.SerializeToString,
            ),
    }
    generic_handler = grpc.method_handlers_generic_handler(
            'social.explore.ExploreService', rpc_method_handlers)
    server.add_generic_rpc_handlers((generic_handler,))


 # This class is part of an EXPERIMENTAL API.
class ExploreService(object):
    """Missing associated documentation comment in .proto file."""

    @staticmethod
    def Ping(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/social.explore.ExploreService/Ping',
            services_dot_common_dot_common__pb2.Empty.SerializeToString,
            services_dot_common_dot_common__pb2.Pong.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def GetFeed(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/social.explore.ExploreService/GetFeed',
            services_dot_common_dot_common__pb2.Pagination.SerializeToString,
            services_dot_post_dot_post__pb2.PostListResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def GeoSearch(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/social.explore.ExploreService/GeoSearch',
            services_dot_explore_dot_explore__pb2.GeoSearchRequest.SerializeToString,
            services_dot_explore_dot_explore__pb2.GeoSearchResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)



import setuptools

setuptools.setup(
    name="mgo_serializer",
    version="0.0.15",
    author='MosyDev',
    author_email='mostafa.uwsgi@gmail.com',
    description='A MongoEngine Serializer to serialize objects from JSON/BSON to JSON and gRPC',
    url='https://github.com/its0x4d/mgo_serializer',
    packages=setuptools.find_packages(),
    license='GPLv3',
    classifiers=[
        'Programming Language :: Python :: 3',
        'License :: OSI Approved :: GNU General Public License v3 (GPLv3)',
    ]
)



import functools
import inspect
from datetime import datetime

try:
    from collections import Mapping
except ImportError:
    # Python 3.10 Support
    from collections.abc import Mapping

from mgo_serializer.helpers import DictMapper

__all__ = [
    'Empty',
    'BaseField',
    'BooleanField',
    'CharField',
    'IntField',
    'ObjectIdField',
    'CustomField',
    'set_value'
]


class Empty:
    """
    Empty class to be used as a placeholder for None
    """
    pass


def is_simple_callable(obj):
    """
    True if the object is a callable that takes no arguments.
    """
    # Bail early since we cannot inspect built-in function signatures.
    if inspect.isbuiltin(obj):
        raise ValueError(
            'Built-in function signatures are not injectable. '
            'Wrap the function call in a simple, pure Python function.')

    if not (inspect.isfunction(obj) or inspect.ismethod(obj) or isinstance(obj, functools.partial)):
        return False

    sig = inspect.signature(obj)
    params = sig.parameters.values()
    return all(
        param.kind == param.VAR_POSITIONAL or
        param.kind == param.VAR_KEYWORD or
        param.default != param.empty
        for param in params
    )


def get_attribute(instance, attrs):
    """
    Similar to Python's built in `getattr(instance, attr)`,
    but takes a list of nested attributes, instead of a single attribute.

    Also accepts either attribute lookup on objects or dictionary lookups.
    """
    for attr in attrs:
        try:
            if isinstance(instance, Mapping):
                instance = instance[attr]
            else:
                instance = getattr(instance, attr)
        except (IndexError, KeyError, AttributeError):
            return None
        if is_simple_callable(instance):
            try:
                instance = instance()
            except (AttributeError, KeyError) as exc:
                # If we raised an Attribute or KeyError here it'd get treated
                # as an omitted field in `Field.get_attribute()`. Instead, we
                # raise a ValueError to ensure the exception is not masked.
                raise ValueError(
                    'Exception raised in callable attribute "{}"; original exception was: {}'.format(attr, exc))

    return instance


def set_value(dictionary, keys, value):
    """
    Similar to Python's built in `dictionary[key] = value`,
    but takes a list of nested keys instead of a single key.

    set_value({'a': 1}, [], {'b': 2}) -> {'a': 1, 'b': 2}
    set_value({'a': 1}, ['x'], 2) -> {'a': 1, 'x': 2}
    set_value({'a': 1}, ['x', 'y'], 2) -> {'a': 1, 'x': {'y': 2}}
    """
    if not keys:
        dictionary.update(value)
        return

    for key in keys[:-1]:
        if key not in dictionary:
            dictionary[key] = {}
        dictionary = dictionary[key]

    dictionary[keys[-1]] = value


class BaseField(object):
    _auto_creation_counter = 0

    ALLOWED_TYPES = None

    initial = None

    def __init__(self, initial=Empty, proto_field=None, default=None, primary_key=None, custom=False, context=None, db_only=False):
        # Increase the auto creation counter
        self._auto_creation_counter = BaseField._auto_creation_counter
        BaseField._auto_creation_counter += 1

        self.initial = self.initial if (initial is Empty) else initial
        self.proto_field = proto_field
        self.default = default
        self.custom = custom
        self.primary_key = primary_key
        self.db_only = db_only
        self.context = {} if context is None else context

        # These are set up by `.bind()` when the field is added to a serializer.
        self.field_name = None
        self.parent = None
        self.source = None
        self.attributes = None

        self.meta = getattr(self, 'Meta', None)
        self.pb = getattr(self.meta, 'schema', None) if self.meta else None

    def bind(self, field_name, parent):
        self.field_name = field_name
        self.parent = parent

        # self.source should default to being the same as the field name.
        if self.source is None:
            self.source = field_name

        if self.source == '*':
            self.attributes = []
        else:
            self.attributes = self.source.split('.')

    def get_initial(self):
        return self.initial() if callable(self.initial) else self.initial

    def get_value(self, dictionary, instance):
        """
        Given the *incoming* dictionary, return the value for this field
        that should be validated and transformed to a native value.
        """

        # convert the dictionary to mapping to make it easier to work with
        dictionary = DictMapper(dictionary) if isinstance(dictionary, dict) else dictionary

        if isinstance(dictionary, dict):
            if '_id' in dictionary:
                dictionary['id'] = dictionary.pop('_id')

        if self.custom:
            fill_method = getattr(instance, f'get_custom_{self.field_name}', None)
            return fill_method(dictionary) if fill_method else None

        return dictionary[self.field_name] if self.field_name in dictionary and not self.custom else Empty

    def get_attribute(self, instance):
        """
        Given the *outgoing* instance, return the value for this field
        that should be serialized.
        """
        return get_attribute(instance, self.attributes)

    def get_default(self):
        """
        Return the default value for this field.
        """
        return self.default

    def validate_empty_values(self, data):
        """
        Validate empty values, and either:
        """
        if data is Empty:
            return True, self.get_default()

        if data is None:
            if self.source == '*':
                return False, None
            return True, None

        return False, data

    def run_validation(self, data=Empty):
        (is_empty_value, data) = self.validate_empty_values(data)
        if is_empty_value:
            return data

        return self.to_internal_value(data)

    def to_internal_value(self, data):
        """
        Given the *incoming* primitive data, return the native value.
        """
        raise NotImplementedError('`to_internal_value()` must be implemented.')

    def to_representation(self, value):
        """
        Given the native value, return the representation of this field
        that should be returned to the user.
        """
        raise NotImplementedError('`to_representation()` must be implemented.')

    def to_protobuf(self, data):
        """
        Given the native value, return the protobuf value.
        """
        raise NotImplementedError('`to_protobuf()` must be implemented.')

    @property
    def root(self):
        """
        Returns the top-level serializer for this field.
        """
        root = self
        while root.parent is not None:
            root = root.parent
        return root

    def __new__(cls, *args, **kwargs):
        instance = super().__new__(cls)
        instance._args = args
        instance._kwargs = kwargs
        return instance


class BooleanField(BaseField):
    TRUE_VALUES = ['t', 'T', 'true', 'True', 'TRUE', '1', 1, True]
    FALSE_VALUES = ['f', 'F', 'false', 'False', 'FALSE', '0', 0, False]
    NULL_VALUES = ['n', 'N', 'null', 'Null', 'NULL', '', None]

    def to_internal_value(self, data):
        if data is not Empty:
            if data in self.TRUE_VALUES:
                return True
            elif data in self.FALSE_VALUES:
                return False
            elif data in self.NULL_VALUES:
                return None
            raise ValueError('Invalid boolean value.')
        return data

    def to_representation(self, value):
        if value in self.TRUE_VALUES:
            return True
        elif value in self.FALSE_VALUES:
            return False
        elif value in self.NULL_VALUES:
            return None
        return bool(value)

    def to_protobuf(self, value):
        _repr = self.to_representation(value)
        if _repr is None:
            return None
        return _repr


class CharField(BaseField):

    def __init__(self, **kwargs):
        self.trim_whitespace = kwargs.pop('trim_whitespace', True)
        super().__init__(**kwargs)

    def to_internal_value(self, data):
        value = str(data)
        return value.strip() if self.trim_whitespace else value

    def to_representation(self, value):
        return str(value) if value is not None else None

    def to_protobuf(self, value):
        _repr = self.to_representation(value)
        if _repr is None:
            return None
        return _repr


class IntField(BaseField):

    def to_internal_value(self, data):
        try:
            return int(data)
        except (TypeError, ValueError):
            raise ValueError('Expected a number.')

    def to_representation(self, value):
        return int(value) if value is not None else None

    def to_protobuf(self, value):
        _repr = self.to_representation(value)
        if _repr is None:
            return None
        return _repr


class FloatField(BaseField):

    def to_internal_value(self, data):
        try:
            return float(data)
        except (TypeError, ValueError):
            raise ValueError('Expected a number.')

    def to_representation(self, value):
        return float(value) if value is not None else None

    def to_protobuf(self, value):
        _repr = self.to_representation(value)
        if _repr is None:
            return None
        return _repr


class ObjectIdField(BaseField):

    def to_internal_value(self, data):
        return str(data)

    def to_representation(self, value):
        return str(value)

    def to_protobuf(self, value):
        _repr = self.to_representation(value)
        if _repr is None:
            return None
        return _repr


class CustomField(BaseField):

    def __init__(self, child=None):
        self.child = child
        super().__init__(custom=True)

    def to_internal_value(self, data):
        if data is not Empty:
            if not self.child:
                return data

            if isinstance(data, list):
                return [self.child.to_internal_value(item) for item in data]
            return self.child.to_internal_value(data)
        return data

    def to_representation(self, value):
        if not self.child:
            return value
        if isinstance(value, list):
            [self.child.to_representation(item) for item in value]
        return self.child.to_representation(value)

    def to_protobuf(self, value):
        _repr = self.to_representation(value)
        if _repr is None:
            return None
        return _repr


class ListField(BaseField):

    ALLOWED_TYPES = [
        'CharField', 'DateTimeField', 'IntField',
        'ObjectIdField', 'CustomField', 'BooleanField',
        'FloatField'
    ]

    def __init__(self, child=None, **kwargs):
        self.type = child
        if child and child.__class__.__name__ not in self.ALLOWED_TYPES:
            raise ValueError(
                'type {!r} is not allowed for {!r}. '
                'Allowed types are: {}'.format(
                    child.__class__.__name__,
                    self.__class__.__name__,
                    ', '.join(self.ALLOWED_TYPES)
                )
            )
        super().__init__(**kwargs)

    def to_internal_value(self, data):
        if data is not Empty:
            if not self.type:
                return data
            return [self.type.to_internal_value(item) for item in data]
        return data

    def to_representation(self, value):
        if not self.type:
            return value
        return [self.type.to_representation(item) for item in value]

    def to_protobuf(self, value):
        _repr = self.to_representation(value)
        if _repr is None:
            return None
        return _repr


class DictField(BaseField):

    ALLOWED_TYPES = ['ListField']

    def __init__(self, child=None, **kwargs):
        self.type = child
        if child and child.__class__.__name__ not in self.ALLOWED_TYPES:
            raise ValueError(
                'type {!r} is not allowed for {!r}. '
                'Allowed types are: {}'.format(
                    child.__class__.__name__,
                    self.__class__.__name__,
                    ', '.join(self.ALLOWED_TYPES)
                )
            )

        super().__init__(**kwargs)

    def to_internal_value(self, data):
        if data is not Empty:
            if not self.type:
                return data
            return {
                key: self.type.to_internal_value(value)
                for key, value in data.items()
            }
        return data

    def to_representation(self, value):
        if not self.type:
            return value
        return {
            key: self.type.to_representation(item)
            for key, item in value.items()
        }

    def to_protobuf(self, value):
        _repr = self.to_representation(value)
        if _repr is None:
            return None
        return _repr


class DateTimeField(BaseField):

    def __init__(self, fmt='%Y-%m-%dT%H:%M:%S', **kwargs):
        self.format = fmt
        super().__init__(**kwargs)

    def to_internal_value(self, data):
        if data is not Empty:
            if isinstance(data, datetime):
                return data.strftime(self.format) if self.format else data
            if self.format:
                data = datetime.strptime(data, self.format)
            else:
                data = datetime.fromtimestamp(data)
            return data
        return data

    def to_representation(self, value):
        if value is not None:
            if self.format and isinstance(value, datetime):
                return value.strftime(self.format)
            return value
        return value

    def to_protobuf(self, value):
        _repr = self.to_representation(value)
        if _repr is None:
            return None
        return _repr


class TimestampField(BaseField):

    ALLOWED_TYPES = ['IntField', 'CharField', 'FloatField']

    def __init__(self, child=None, **kwargs):
        self.type = IntField() if child is None else child
        if child and child.__class__.__name__ not in self.ALLOWED_TYPES:
            raise ValueError(
                'type {!r} is not allowed for {!r}. '
                'Allowed types are: {}'.format(
                    child.__class__.__name__,
                    self.__class__.__name__,
                    ', '.join(self.ALLOWED_TYPES)
                )
            )
        super().__init__(**kwargs)

    def to_internal_value(self, data):
        if data is not Empty:
            if isinstance(data, datetime):
                return data.timestamp()
            return self.type.to_internal_value(data)
        return data

    def to_representation(self, value):
        if value is not None:
            if isinstance(value, datetime):
                return value.timestamp()
            return self.type.to_representation(value)
        return value

    def to_protobuf(self, value):
        _repr = self.to_representation(value)
        if _repr is None:
            return None
        return _repr
